#include "aabbox.h"
#include <GL/gl.h>

using namespace std;
using namespace qglviewer;

namespace
{
  const unsigned short faceCorners[6][4] =
    {
      { 0,4,6,2 },
      { 5,1,3,7 },
      { 0,1,5,4 },
      { 3,2,6,7 },
      { 0,2,3,1 },
      { 6,4,5,7 }
    };
  const Vec normals[6] =
    {
      Vec(-1.0f, 0.0f, 0.0f),
      Vec( 1.0f, 0.0f, 0.0f),
      Vec( 0.0f,-1.0f, 0.0f),
      Vec( 0.0f, 1.0f, 0.0f),
      Vec( 0.0f, 0.0f,-1.0f),
      Vec( 0.0f, 0.0f, 1.0f)
    };
}
//************************************************************
// Implementation of AABBox
//************************************************************
/*! \class AABBox
 *
 * \ingroup BBVOLUMES
 *
 * A class to hold an axis aligned bounding box in 3D.
 */
/*! \fn Vec AABBox::corner(const unsigned short i) const
 *
 * Returns the vertices of the bbox in a logical order (decomposition of i
 * in binary gives position along axies).
 */
void
AABBox::drawUnlit() const
{
  const Vec corners[8] =
    {
      corner(0),
      corner(1),
      corner(2),
      corner(3),
      corner(4),
      corner(5),
      corner(6),
      corner(7)
    };
  glBegin(GL_QUADS);
  for (unsigned short f=0;f<6;++f)
  {
    const unsigned short* face = faceCorners[f];
    glVertex3fv(corners[face[0]]);
    glVertex3fv(corners[face[1]]);
    glVertex3fv(corners[face[2]]);
    glVertex3fv(corners[face[3]]);
  }
  glEnd();
}
void
AABBox::drawLit() const
{
  const Vec corners[8] =
    {
      corner(0),
      corner(1),
      corner(2),
      corner(3),
      corner(4),
      corner(5),
      corner(6),
      corner(7)
    };
  glBegin(GL_QUADS);
  for (unsigned short f=0;f<6;++f)
  {
    glNormal3fv(normals[f]);
    const unsigned short* face = faceCorners[f];
    glVertex3fv(corners[face[0]]);
    glVertex3fv(corners[face[1]]);
    glVertex3fv(corners[face[2]]);
    glVertex3fv(corners[face[3]]);
  }
  glEnd();
}
//************************************************************
// Implementation of AABBox::Face
//************************************************************
AABBox::Face::Face(const AABBox* b,
                   const unsigned short pos)
  : aabbox_(b),
    pos_(pos),
    corners_(faceCorners[pos])
{
}
